@@ -1,15 +1,30 @@
+#    Copyright 2021 SolarWinds, LLC
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """AppOptics APM instrumentation for Django
-
- Copyright (C) 2016 by SolarWinds, LLC.
- All rights reserved.
 """
 import functools
+import importlib
+import logging
+import sys
 import threading
 import time
 
-from appoptics_apm import config, imports, util
+from appoptics_apm import config, util
 from appoptics_apm.transaction_filter import UrlGetter
 
+logger = logging.getLogger(__name__)
+
 try:
     from django.utils.deprecation import MiddlewareMixin as AppOpticsApmMiddlewareBase
 except ImportError:
@@ -18,7 +33,18 @@ except ImportError:
 # django middleware for passing values to appoptics_apm
 __all__ = ("AppOpticsApmDjangoMiddleware", "install_appoptics_apm_middleware")
 
-appoptics_apm_logger = util.logger
+
+def _import_and_wrap(module_name, hook=None):
+    """ Import the module provided in module_name and execute the provided hook on the imported module. hook is expected
+    to be a callable receiving a module as the only argument and should take care of wrapping the necessary objects of
+    the provided module."""
+
+    try:
+        importlib.import_module(module_name)
+    except ImportError as error:
+        logger.debug("Failed to import %s with %s", module_name, error)
+        return
+    hook(sys.modules[module_name])
 
 
 class DjangoUrlGetter(UrlGetter):
@@ -32,14 +58,17 @@ class DjangoUrlGetter(UrlGetter):
 
 
 class AppOpticsApmWSGIHandler(object):
-    """ Wrapper WSGI Handler for Django's django.core.handlers.wsgi:WSGIHandler
+    """Wrapper WSGI Handler for Django's django.core.handlers.wsgi:WSGIHandler.
     Can be used as a replacement for Django's WSGIHandler, e.g. with uWSGI.
     """
     def __init__(self):
         """ Import and instantiate django.core.handlers.WSGIHandler,
         now that the load_middleware wrapper below has been initialized. """
-        from django.core.handlers.wsgi import WSGIHandler
-        self._handler = WSGIHandler()
+        from django.core.wsgi import get_wsgi_application
+        self._handler = get_wsgi_application()
+        logger.warning(
+            "AppOpticsApmWSGIHandler class is obsolete. Please refer to the AppOptics documentation on " +
+            "how to set up instrumentation of Django applications.")
 
     def __call__(self, environ, start_response):
         return self._handler(environ, start_response)
@@ -68,11 +97,10 @@ class AppOpticsApmDjangoMiddleware(AppOpticsApmMiddlewareBase):
                 setattr(util.Context.transaction_dict, 'url_tran', url.get_url())
                 if ctx.is_sampled():
                     ctx.report(start_evt)
-                    ctx.set_as_default()
             request.META.setdefault('APPOPTICS_APM_SPAN_START', str(int(time.time() * 1000000)))
 
         except Exception as e:
-            appoptics_apm_logger.error("AppOptics APM middleware error: %s" % self._singleline(e))
+            logger.error("AppOptics APM middleware error: %s" % self._singleline(e))
 
     def process_view(self, request, view_func, view_args, view_kwargs):
         ctx = util.Context.get_default()
@@ -90,9 +118,9 @@ class AppOpticsApmDjangoMiddleware(AppOpticsApmMiddlewareBase):
                     'Controller': controller,
                     'Action': action,
                 }
-                util.log('process_view', None, keys=kvs, store_backtrace=False)
+                util.log('info', None, keys=kvs, store_backtrace=False)
         except Exception as e:
-            appoptics_apm_logger.error("AppOptics APM middleware error: %s" % self._singleline(e))
+            logger.error("AppOptics APM middleware error: %s" % self._singleline(e))
 
     def process_response(self, request, response):
         """Process the response, record some information and send the end_trace out
@@ -102,14 +130,13 @@ class AppOpticsApmDjangoMiddleware(AppOpticsApmMiddlewareBase):
         try:
             if ctx:
                 # even if the request is not sampled, we need to gather this information to report metrics
-                transaction_name = ctx.get_transaction_name()
-                if not transaction_name:
-                    transaction_name = request.META.pop('APPOPTICS_APM_SPAN_TRANSACTION', None)
+                if not ctx.get_transaction_name():
+                    ctx.set_transaction_name(request.META.pop('APPOPTICS_APM_SPAN_TRANSACTION', None))
                 # exit event needs it. will be processed there
-                domain = request.META.get('HTTP_X_FORWARDED_HOST', request.META.get('HTTP_HOST', 'localhost'))
+                setattr(
+                    util.Context.transaction_dict, 'forwarded_host', request.META.get('HTTP_X_FORWARDED_HOST', None))
 
-                setattr(util.Context.transaction_dict, 'domain', domain)
-                setattr(util.Context.transaction_dict, 'transaction_name', transaction_name)
+                setattr(util.Context.transaction_dict, 'domain', request.META.get('HTTP_HOST', 'localhost'))
                 setattr(util.Context.transaction_dict, 'start_time', request.META.pop('APPOPTICS_APM_SPAN_START', None))
                 setattr(util.Context.transaction_dict, 'request_method', request.META.get('REQUEST_METHOD', 'GET'))
                 setattr(util.Context.transaction_dict, 'status_code', response.status_code)
@@ -117,9 +144,9 @@ class AppOpticsApmDjangoMiddleware(AppOpticsApmMiddlewareBase):
             x_trace = util.end_http_trace(self.layer)
             if x_trace:
                 response['X-Trace'] = x_trace
-                appoptics_apm_logger.debug("djangoware process_response x_trace: {x}".format(x=x_trace))
+                logger.debug("djangoware process_response x_trace: {x}".format(x=x_trace))
         except Exception as e:
-            appoptics_apm_logger.error("AppOptics APM middleware error: %s" % self._singleline(e))
+            logger.error("AppOptics APM middleware error: %s" % self._singleline(e))
 
         return response
 
@@ -127,14 +154,14 @@ class AppOpticsApmDjangoMiddleware(AppOpticsApmMiddlewareBase):
         try:
             util.log_exception()
         except Exception as e:
-            appoptics_apm_logger.error("AppOptics APM middleware error: %s" % self._singleline(e))
+            logger.error("AppOptics APM middleware error: %s" % self._singleline(e))
 
 
 def middleware_hooks(module, objname):
     try:
         # wrap middleware callables we want to wrap
         cls = getattr(module, objname, None)
-        appoptics_apm_logger.info('middleware_hooks module {m}'.format(m=module))
+        logger.info('middleware_hooks module {m}'.format(m=module))
         if not cls:
             return
         for method in ['process_request',
@@ -148,7 +175,7 @@ def middleware_hooks(module, objname):
             profile_name = '%s.%s.%s' % (module.__name__, objname, method)
             setattr(cls, method, util.profile_function(profile_name)(fn))
     except Exception as e:
-        appoptics_apm_logger.error("AppOptics APM error: %s" % str(e))
+        logger.error("AppOptics APM error: %s" % str(e))
 
 
 load_middleware_lock = threading.Lock()
@@ -193,27 +220,23 @@ def on_load_middleware():
             if dot < 0 or dot + 1 == len(i):
                 continue
             objname = i[dot + 1:]
-            imports.whenImported(i[:dot], functools.partial(middleware_hooks, objname=objname))
+            _import_and_wrap(i[:dot], functools.partial(middleware_hooks, objname=objname))
         # ORM
         if config['inst_enabled']['django_orm']:
             from appoptics_apm import inst_django_orm
-            # The wrapper path BaseDatabaseWrapper has changed in Django 1.8 and onwards.
+            # The wrapper path BaseDatabaseWrapper has changed in Django 1.8 and onwards. Thus, we need to import here
+            # so we are able to catch the ImportError depending on the running Django version
             try:
                 import django.db.backends.base.base  # pylint: disable-msg=unused-import
-                imports.whenImported('django.db.backends.base.base', inst_django_orm.wrap)
+                _import_and_wrap('django.db.backends.base.base', inst_django_orm.wrap)
             except ImportError as e:
-                appoptics_apm_logger.debug('AppOptics on_load_middleware: {e}, try loading dummy.base'.format(e=e))
-                try:
-                    import django.db.backends.dummy.base
-                    imports.whenImported('django.db.backends.dummy.base', inst_django_orm.wrap)
-                except ImportError as e:
-                    appoptics_apm_logger.error('AppOptics error in on_load_middleware: {e}'.format(e=e))
+                logger.debug('AppOptics on_load_middleware: {e}, try loading dummy.base'.format(e=e))
+                _import_and_wrap('django.db.backends.dummy.base', inst_django_orm.wrap)
 
         # templates
         if config['inst_enabled']['django_templates']:
             from appoptics_apm import inst_django_templates
-            import django
-            imports.whenImported('django.template.base', inst_django_templates.wrap)
+            _import_and_wrap('django.template.base', inst_django_templates.wrap)
 
         # load pluggable instrumentation
         from .loader import load_inst_modules
@@ -227,19 +250,19 @@ def on_load_middleware():
             else:
                 settings.MIDDLEWARE_CLASSES = (apm_middleware, ) + middleware
         else:
-            appoptics_apm_logger.error(
+            logger.error(
                 "AppOptics APM error: settings middleware attribute should be either a tuple or a list,"
                 "got {mw_type}".format(mw_type=str(type(middleware))))
     except Exception as e:
-        appoptics_apm_logger.error('AppOptics APM error in on_load_middleware: {e}'.format(e=e))
+        logger.error('AppOptics APM error in on_load_middleware: {e}'.format(e=e))
 
     finally:  # release instrumentation lock
         mwlock.release()
 
 
-def install_appoptics_apm_middleware(module):
+def install_appoptics_apm_middleware():
     def base_handler_wrapper(func):
-        @functools.wraps(func)  # XXX Not Python2.4-friendly
+        @functools.wraps(func)
         def wrap_method(*f_args, **f_kwargs):
             on_load_middleware()
             return func(*f_args, **f_kwargs)
@@ -247,23 +270,27 @@ def install_appoptics_apm_middleware(module):
         return wrap_method
 
     try:
-        cls = getattr(module, 'BaseHandler', None)
-        try:
-            if not cls or cls.APPOPTICS_APM_MIDDLEWARE_LOADER:
-                return
-        except AttributeError as e:
-            cls.APPOPTICS_APM_MIDDLEWARE_LOADER = True
+        import django.core.handlers.base
+
+        # wrap Django load_middleware method
+        cls = getattr(django.core.handlers.base, 'BaseHandler', None)
+        if cls is None:
+            raise Exception('Could not load Django BaseHandler class')
+        if getattr(cls, 'APPOPTICS_APM_MIDDLEWARE_LOADER', None):
+            return
+        # add attribute to prevent class from being wrapped multiple times
+        cls.APPOPTICS_APM_MIDDLEWARE_LOADER = True
         fn = getattr(cls, 'load_middleware', None)
+        if not callable(fn):
+            raise Exception('Could not locate load_middleware method')
         setattr(cls, 'load_middleware', base_handler_wrapper(fn))
+
+        util.report_layer_init(layer='django')
     except Exception as e:
-        appoptics_apm_logger.error("AppOptics APM error: %s" % str(e))
+        # gracefully exit if Django lib not present or loaded properly
+        logger.error("Unable to load and instrument Django: %s" % e)
+        return
 
 
 if util.ready():
-    try:
-        imports.whenImported('django.core.handlers.base', install_appoptics_apm_middleware)
-        # phone home
-        util.report_layer_init(layer='django')
-    except ImportError as e:
-        # gracefully disable tracing if AppOptics APM lib not present
-        appoptics_apm_logger.error("[AppOptics] Unable to instrument app and middleware: %s" % e)
+    install_appoptics_apm_middleware()
