@@ -1,13 +1,24 @@
+#    Copyright 2021 SolarWinds, LLC
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ AppOptics APM instrumentation for Django ORM.
-
-Copyright (C) 2016 by SolarWinds, LLC.
-All rights reserved.
 """
+import logging
 import re
 
 from appoptics_apm import config, util
 
-appoptics_apm_logger = util.logger
+logger = logging.getLogger(__name__)
 
 
 def wrap_execute(func, f_args, f_kwargs, res):
@@ -86,7 +97,7 @@ def wrap(module):
             try:
                 return CursorAppOpticsApmWrapper(cursor_method(self), self)
             except Exception as e:
-                appoptics_apm_logger.error("[AppOptics] Error in cursor_wrap: %s" % e)
+                logger.error("[AppOptics] Error in cursor_wrap: %s" % e)
                 # need to re-raise application exceptions thrown here
                 raise
 
@@ -94,4 +105,4 @@ def wrap(module):
 
         setattr(module.BaseDatabaseWrapper, 'cursor', cursor_wrap)
     except Exception as e:
-        appoptics_apm_logger.error("[AppOptics] Error in module_wrap: %s" % e)
+        logger.error("[AppOptics] Error in module_wrap: %s" % e)
