@@ -1,11 +1,23 @@
+#    Copyright 2021 SolarWinds, LLC
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ AppOptics APM instrumentation for Django's template system
-
-Copyright (C) 2016 SolarWinds, LLC.
-All rights reserved.
 """
+import logging
+
 from appoptics_apm import util
 
-appoptics_apm_logger = util.logger
+logger = logging.getLogger(__name__)
 
 TEMPLATE_LAYER = 'template'
 
@@ -43,4 +55,4 @@ def wrap(module):
         setattr(module.Template, 'render', layer_wrapper_render(profile_wrapper_render(module.Template.render)))
 
     except Exception as e:
-        appoptics_apm_logger.error("AppOptics APM error: %s" % str(e))
+        logger.error("AppOptics APM error: %s" % str(e))
