@@ -1,7 +1,17 @@
+#    Copyright 2021 SolarWinds, LLC
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ AppOptics APM instrumentation for logging module.
-
-Copyright (C) 2020 by SolarWinds, LLC.
-All rights reserved.
 """
 import inspect
 import logging
@@ -10,6 +20,8 @@ from decorator import decorator
 
 import appoptics_apm
 
+ao_logger = logging.getLogger(__name__)
+
 _INJECTION_STRING = 'ao.traceId={}'
 
 
@@ -95,13 +107,11 @@ class LogInjector(object):
                 fn = getattr(logger, f_name, None)
                 if callable(fn):
                     fn(self.injection_filter)
-                    appoptics_apm.util.logger.debug(
-                        'update_filters: logger {}, {}(LogInjectionFilter).'.format(logger, f_name))
+                    ao_logger.debug('update_filters: logger {}, {}(LogInjectionFilter).'.format(logger, f_name))
                 else:
-                    appoptics_apm.util.logger.warning(
-                        'update_filters: could not call {} on logger {}.'.format(f_name, logger))
+                    ao_logger.warning('update_filters: could not call {} on logger {}.'.format(f_name, logger))
         except Exception as e:
-            appoptics_apm.util.logger.error("AppOptics APM error: %s" % str(e))
+            ao_logger.error("AppOptics APM error: %s" % str(e))
         finally:
             logging._releaseLock()
 
@@ -117,9 +127,9 @@ def wrap_logger_init(func, self, *f_args, **f_kwargs):
     try:
         if log_injector.is_injecting():
             self.addFilter(log_injector.injection_filter)
-            appoptics_apm.util.logger.debug('wrap_logger_init adding LogInjectionFilter to logger {}.'.format(self))
+            ao_logger.debug('wrap_logger_init adding LogInjectionFilter to logger {}.'.format(self))
     except Exception as e:
-        appoptics_apm.util.logger.error(
+        ao_logger.error(
             "AppOptics APM error: wrap_logger_init could not add LogInjectionFilter and received following error: {}".
             format(e))
 
@@ -132,12 +142,11 @@ def wrap_logging_module():
             if hasattr(fn, '__func__'):  # Is this a bound method of an object
                 fn = fn.__func__  # then wrap the unbound method
             setattr(logger, '__init__', decorator(wrap_logger_init, fn))
-            appoptics_apm.util.logger.debug(
-                'wrap_logging_module wraps {} of {} object'.format(fn.__name__, logger.__name__))
+            ao_logger.debug('wrap_logging_module wraps {} of {} object'.format(fn.__name__, logger.__name__))
             log_injector.set_injection_mode(appoptics_apm.config['log_trace_id'])
             log_injector.set_wrapped()
         except Exception as e:
-            appoptics_apm.util.logger.error("AppOptics APM error: %s" % str(e))
+            ao_logger.error("AppOptics APM error: %s" % str(e))
 
 
 def get_log_trace_id():
