@@ -1,11 +1,23 @@
+#    Copyright 2021 SolarWinds, LLC
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ AppOptics APM instrumentation for SQLAlchemy.
-
- Copyright (C) 2016 by SolarWinds, LLC.
- All rights reserved.
 """
+import logging
+
 from appoptics_apm import config, util
 
-appoptics_apm_logger = util.logger
+logger = logging.getLogger(__name__)
 
 
 def main():
@@ -51,7 +63,7 @@ def wrap_methods(cls, mappings):
         try:
             base_method = getattr(cls, name)
         except AttributeError:
-            appoptics_apm_logger.error('Failed to patch %s on %s', name, cls.__name__)
+            logger.error('Failed to patch %s on %s', name, cls.__name__)
         else:
             appoptics_apm_fn = util.log_method(
                 'sqlalchemy', store_backtrace=util._collect_backtraces('sqlalchemy'), callback=fn)(base_method)
