@@ -1,8 +1,19 @@
+#    Copyright 2021 SolarWinds, LLC
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ WSGI middleware for appoptics_apm support
-
-Copyright (C) 2016 by SolarWinds, LLC.
-All rights reserved.
 """
+import logging
 import sys
 import time
 import traceback as tb
@@ -13,6 +24,8 @@ from appoptics_apm import config, util
 from appoptics_apm.loader import load_inst_modules
 from appoptics_apm.transaction_filter import UrlGetter
 
+logger = logging.getLogger(__name__)
+
 
 class WsgiUrlGetter(UrlGetter):
     def __init__(self, environ):
@@ -91,7 +104,7 @@ class AppOpticsApmMiddleware(object):
             config['sample_rate'] = float(self.appoptics_apm_config['appoptics_apm.sample_rate'])
 
         if 'reporter_host' in self.appoptics_apm_config or 'reporter_port' in self.appoptics_apm_config:
-            util.logger.warning("Ignore deprecated configuration options: reporter_host and reporter_port.")
+            logger.warning("Ignore deprecated configuration options: reporter_host and reporter_port.")
 
         # load pluggable instrumentation
         load_inst_modules()
@@ -116,20 +129,21 @@ class AppOpticsApmMiddleware(object):
         if ctx.is_sampled():
             # get some HTTP details from WSGI vars
             # http://www.wsgi.org/en/latest/definitions.html
-            for hosthdr in ("HTTP_HOST", "HTTP_X_HOST", "HTTP_X_FORWARDED_HOST", "SERVER_NAME"):
-                if hosthdr in environ:
-                    start_evt.add_info("HTTP-Host", environ[hosthdr])
-                    break
             if 'PATH_INFO' in environ:
                 start_evt.add_info("URL", environ['PATH_INFO'])
-            if 'REQUEST_METHOD' in environ:
-                start_evt.add_info("Method", environ['REQUEST_METHOD'])
             if 'QUERY_STRING' in environ:
                 start_evt.add_info("Query-String", environ['QUERY_STRING'])
 
             ctx.report(start_evt)
-            ctx.set_as_default()
-            end_evt = ctx.create_event('exit', self.layer)
+            # Creates a new event with the same task ID as current context, a new op_id and no edges. This is a slightly
+            # hacky approach since the Oboe wrapper does not expose constructors for the Event class.
+            # This manual event generation is necessary as Oboe would automatically add an edge to the previous event if
+            # ctx.create_event is used. However, at this point, we do not yet know the event ID this end event needs to
+            # be linked to.
+            # Note that the pre-generation of the end event is necessary in order to inject the X-Trace ID into the
+            # header in wrapped_start_response (rather than in __calls__'s finally block) before calling the original
+            # start_response method as per WSGI middleware convention.
+            end_evt = util.Event(util.SwigEvent.startTrace(ctx._md), 'exit', self.layer)
 
         response_body = []
 
@@ -140,7 +154,6 @@ class AppOpticsApmMiddleware(object):
             status_str[0] = status
 
             if ctx.is_sampled():
-                end_evt.add_info("Status", status.split(' ', 1)[0])
                 if exc_info:
                     _t, exc, trace = exc_info
                     end_evt.add_info("ErrorMsg", str(exc))
@@ -157,7 +170,6 @@ class AppOpticsApmMiddleware(object):
         stats = None
         result = None
         exc_thrown = False
-        domain = None
 
         try:
             if self.profile and ctx.is_sampled():
@@ -189,7 +201,7 @@ class AppOpticsApmMiddleware(object):
                     stats = sio.getvalue()
                     sio.close()
                 except Exception as aoe:  # catch possible non-application thrown exception
-                    util.logger.error(
+                    logger.error(
                         'AppOpticsMiddleware cProfile, pstats'
                         'processing encountered problem:: {}'.format(aoe))
             else:
@@ -197,66 +209,68 @@ class AppOpticsApmMiddleware(object):
 
         except Exception as e:
             exc_thrown = True
-            util.logger.error('Error in AppOpticsApmMiddleware: {}'.format(e))
+            logger.error('Error in AppOpticsApmMiddleware: {}'.format(e))
             raise  # re-throw wrapped app raised exception
 
         finally:
-            trans_name = util.get_transaction_name()
-            if end_evt:
-                # check current TLS context and add to end event if valid
-                if util.Context.get_default().is_sampled():
-                    end_evt.add_edge(util.Context.get_default())
 
             #domain and url_tran are required for creating spans
             stats = None if exc_thrown else stats
-            setattr(util.Context.transaction_dict, 'status_code', int(status_str[0].split()[0]))
-            if config['transaction']['prepend_domain_name']:
-                domain = environ.get('HTTP_X_FORWARDED_HOST', environ.get('HTTP_HOST'))
-            setattr(util.Context.transaction_dict, 'url_tran', None if trans_name else url.get_url())
-
+            setattr(util.Context.transaction_dict, 'status_code', int(status_str[0].split(' ')[0]))
+            setattr(util.Context.transaction_dict, 'url_tran', url.get_url())
             setattr(util.Context.transaction_dict, 'http', True)
-            setattr(util.Context.transaction_dict, 'domain', domain)
-            setattr(util.Context.transaction_dict, 'transaction_name', trans_name)
+            setattr(util.Context.transaction_dict, 'domain', environ.get('HTTP_HOST'))
+            setattr(util.Context.transaction_dict, 'forwarded_host', environ.get('HTTP_X_FORWARDED_HOST', None))
             setattr(util.Context.transaction_dict, 'start_time', start_time)
             setattr(util.Context.transaction_dict, 'request_method', environ.get('REQUEST_METHOD'))
-            self.send_end(ctx, end_evt, environ, threw_error=exc_thrown, stats=stats)
+            # send_end needs the current context (to have access to the last reported event id) so that the end event
+            # can be linked to the last reported event.
+            self.send_end(util.Context.get_default(), end_evt, environ, threw_error=exc_thrown, stats=stats)
 
         return result
 
     @classmethod
     def send_end(cls, ctx, evt, environ, threw_error=None, stats=None):
         if ctx is None:
-            util.logger.warning('ctx is None')
+            logger.warning('ctx is None')
+            return
+
+        if not ctx.is_sampled():
+            # even though ctx is not sampled, the trace must still be ended to report metrics
+            ctx.end_trace(evt)
             return
 
         try:
-            if ctx.is_sampled():
-                evt.add_edge(ctx)
-                util.logger.debug('AppOpticsMiddleware send event: {e}'.format(e=evt))
-                if stats:
-                    evt.add_info("Profile", stats)
-                if threw_error:
-                    _t, exc, trace = sys.exc_info()
-                    evt.add_info("ErrorMsg", str(exc))
-                    evt.add_info("ErrorClass", exc.__class__.__name__)
-                    evt.add_info("Backtrace", "".join(tb.format_list(tb.extract_tb(trace))))
-                    del trace  # delete reference to traceback object to allow garbage collection
+            # We need to manually add an edge (pointing to the last reported event) since the pre-generated event does
+            # not have any edges attached.
+            evt.add_edge(ctx)
+            logger.debug('AppOpticsMiddleware send event: {e}'.format(e=evt))
+            if stats:
+                evt.add_info("Profile", stats)
+            if threw_error:
+                _t, exc, trace = sys.exc_info()
+                evt.add_info("ErrorMsg", str(exc))
+                evt.add_info("ErrorClass", exc.__class__.__name__)
+                evt.add_info("Backtrace", "".join(tb.format_list(tb.extract_tb(trace))))
+                del trace  # delete reference to traceback object to allow garbage collection
 
-                # gets controller, action
-                for k, v in environ.get('wsgiorg.routing_args', [{}, {}])[1].items():
-                    if k == "action":
-                        evt.add_info(str(k).capitalize(), str(v))
-                    elif k == "controller":
-                        try:
-                            # handle cases used in openstack's WSGI (and possibly others)
-                            if v.controller:
-                                evt.add_info(str(k).capitalize(), str(v.controller.__class__.__name__))
-                            else:
-                                evt.add_info(str(k).capitalize(), str(v))
-                        except Exception:
-                            evt.add_info(str(k).capitalize(), str(v))
+            # gets controller, action
+            action = None
+            controller = None
+            for k, v in environ.get('wsgiorg.routing_args', [{}, {}])[1].items():
+                if k == "action":
+                    action = str(v)
+                    evt.add_info(str(k).capitalize(), action)
+                elif k == "controller":
+                    try:
+                        # handle cases used in openstack's WSGI (and possibly others)
+                        controller = str(v.controller.__class__.__name__)
+                    except Exception:
+                        controller = str(v)
+                    evt.add_info(str(k).capitalize(), controller)
+            if not ctx.get_transaction_name() and action and controller:
+                ctx.set_transaction_name('{controller}.{action}'.format(controller=controller, action=action))
 
         finally:
-            # report, then clear trace context now that trace is over. Either ctx is_sampled or not,
-            # end_trace must be called to create spans
+            # report, then clear trace context now that trace is over.
             ctx.end_trace(evt)
