@@ -1,12 +1,23 @@
+#    Copyright 2021 SolarWinds, LLC
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ AppOptics APM instrumentation for Tornado.
-
-Copyright (C) 2016 by SolarWinds, LLC.
-All rights reserved.
 """
 # useful methods for instrumenting Tornado
 from __future__ import with_statement
 
 import functools
+import logging
 import time
 
 import tornado.web
@@ -14,7 +25,7 @@ import tornado.web
 from appoptics_apm import aoasync, util
 from appoptics_apm.transaction_filter import UrlGetter
 
-appoptics_apm_logger = util.logger
+logger = logging.getLogger(__name__)
 
 
 # instrumentation functions for tornado.web.RequestHandler
@@ -29,22 +40,16 @@ def request_handler_start(self):
     # check for X-Trace header in HTTP request
     url = UrlGetter(self.request.full_url())
     ctx, evt = util.Context.start_trace('tornado', xtr=self.request.headers.get("X-Trace"), url_getter=url)
+    setattr(util.Context.transaction_dict, 'url_tran', url.get_url())
 
     if ctx.is_sampled() and evt.is_valid():
         if hasattr(self, '__class__') and hasattr(self.__class__, '__name__'):
             evt.add_info("Controller", self.__class__.__name__)
             evt.add_info("Action", self.request.method.lower())
-        evt.add_info("URL", self.request.uri)
-        evt.add_info("Method", self.request.method)
-        evt.add_info("HTTP-Host", self.request.host)
-        setattr(util.Context.transaction_dict, 'url_tran', url.get_url())
         ctx.report(evt)
-        ctx.set_as_default()
     self.request._appoptics_apm_ctx = ctx
-    # XXX
-    self.set_header("X-Trace", str(ctx))
 
-    appoptics_apm_logger.debug('tornado_oboe::request_handler_start trace: {tr}'.format(tr=str(ctx)))
+    logger.debug('tornado_oboe::request_handler_start trace: {tr}'.format(tr=str(ctx)))
     self.request._appoptics_apm_span_start = start_time
 
 
@@ -65,20 +70,20 @@ def request_handler_finish(self):
     try:
         ctx = getattr(self.request, '_appoptics_apm_ctx', util.Context.get_default())
         start_time = getattr(self.request, '_appoptics_apm_span_start', None)
-        transaction_name = ctx.get_transaction_name()
-        if not transaction_name:
-            transaction_name = '{controller}.{action}'.format(
-                controller=self.__class__.__name__, action=self.request.method.lower())
+        if not ctx.get_transaction_name():
+            ctx.set_transaction_name(
+                '{controller}.{action}'.format(controller=self.__class__.__name__, action=self.request.method.lower()))
         setattr(util.Context.transaction_dict, 'domain', self.request.host)
-        setattr(util.Context.transaction_dict, 'transaction_name', transaction_name)
         setattr(util.Context.transaction_dict, 'start_time', start_time)
         setattr(util.Context.transaction_dict, 'request_method', self.request.method)
         setattr(util.Context.transaction_dict, 'status_code', status_code)
 
-        self.set_header("X-Trace", str(ctx))
-        util.end_http_trace(ctx.layer)
+        xtr = util.end_http_trace(ctx.layer)
+        if xtr:
+            # the received Context might have been invalid (i.e. xtr='') in which case we should not inject the x-trace
+            self.set_header("X-Trace", xtr)
     except Exception as e:
-        appoptics_apm_logger.error("AppOptics APM tornado_oboeerror: %s" % str(e))
+        logger.error("AppOptics APM tornado_oboeerror: %s" % str(e))
     # clear the stored appoptics_apm event/metadata from the request object
     self.request._appoptics_apm_ctx = None
 
@@ -166,14 +171,16 @@ class AppOpticsApmBaseHandler(tornado.web.RequestHandler):
         defined handler. If a handler inherits from this base handler, it should call
         this function in its own prepare() function."""
         super(AppOpticsApmBaseHandler, self).prepare()
-        request_handler_start(self)
+        if util.ready():
+            request_handler_start(self)
 
-    def on_finish(self):
+    def finish(self, **kwargs):
         """This function will be called by tornado.web.Application after the user defined
         handler. If a handler inherits from this base handler, it should call this function
         in its own prepare() function."""
-        super(AppOpticsApmBaseHandler, self).on_finish()
-        request_handler_finish(self)
+        if util.ready():
+            request_handler_finish(self)
+        super(AppOpticsApmBaseHandler, self).finish(**kwargs)
 
 
 util.report_layer_init('tornado')
